class PrecisionAnalysis:
    def __init__(self, remote_hostname=None, remote_user=None, remote_scratch=None, job_template=None, vault=None, error_thresholds=None, reduced_precision_level=10, max_running_jobs=10, experiment_name="PrecisionAnalysis", dummy=False, restarting_from_checkpoint=False, assertion_template=None,
                 limit_file=None, analysis_variables=None, forced_variables=None, local_folder=None):
        if dummy is False:
            from UtilsWorkflow.Communicator import SSH

            self.experiment_name = experiment_name
            self.remote_hostname = remote_hostname
            self.remote_user = remote_user
            self.remote_scratch = remote_scratch
            self.communicator = SSH(user=self.remote_user, host=self.remote_hostname, remote_scratch=self.remote_scratch)
            self.job_template = job_template
            self.vault = vault
            assert isinstance(error_thresholds, dict)
            self.error_thresholds = error_thresholds
            self.reduced_precision_level = reduced_precision_level
            self.max_running_jobs = max_running_jobs
            self.restarting_from_checkpoint = restarting_from_checkpoint
            self.assertion_template = assertion_template
            self.analysis_variables = analysis_variables
            self.local_folder = local_folder
            if forced_variables is not None:
                self.forced_variables = forced_variables[:]
            else:
                self.forced_variables = None
            if limit_file is not None:
                import pickle as pkl
                self.limit_file = pkl.load(open(limit_file, "rb"))
            else:
                self.limit_file = None

    def run(self):
        from UtilsWorkflow.WorkflowManagment import BinarySearchJob, Queue
        from Collector.consistency_checks import recover_checkpoint, saving_state, checkpoint
        import time

        vault = self.vault
        # Keeping the variables that have an id (it's precision can be changed in the code)
        variables = [v for v in vault.variables if v.id is not None]

        if self.analysis_variables:
            variables = [v for v in variables if v.id in self.analysis_variables]

        # def shift(l, n):
        #     return l[n:] + l[:n]
        #
        # variables = shift(variables, 15)
        print("Starting analysis with %i variables" % len(variables))
        # Original set including all the variables
        original_set = [v.id for v in variables]

        recovering_from_checkpoint = self.restarting_from_checkpoint
        if not recovering_from_checkpoint:
            # Creating the first set that will include all the variables
            whole_set = BinarySearchJob(analysis_set=original_set, forced_variables=self.forced_variables,
                                        reduced_precision_level=self.reduced_precision_level,
                                        communicator=self.communicator, vault=vault, template=self.job_template,
                                        assertion_template=self.assertion_template, limit_file=self.limit_file,
                                        local_folder=self.local_folder)
            whole_set.thresholds = self.error_thresholds
            # Defining it's level to 0
            whole_set.level = 0
            # Initializing an empty queue for the execution
            queue = Queue()
            # Add the whole set to the queue
            queue.pending.append(whole_set)
            # Init loop counter
            loops = 0
        else:
            queue, whole_set, loops = recover_checkpoint(communicator=self.communicator, vault=vault)
            whole_set.vault = vault
            if self.assertion_template:
                for job in queue.all():
                    if job.assertion_job is not None:
                        job.assertion_job.communicator = self.communicator

        # Define a condition to keep iterating with the refinement
        def condition(_job):
            return _job.status is not "SUCCESS" and _job.status is not "FAILED"
        # Start simulations
        wait = False
        waiting_time = 10
        while condition(whole_set):
            # Main loop
            print("ITERATION: %4i" % loops)
            for key in queue._Queue__dict.keys():
                print("\t%18s: %4i" % (key, len(queue[key])))
            loops += 1

            # Sorting the queue to launch first the lower levels (bigger groups).
            queue.sort(lower_first=True)

            for job in queue.all():
                job.driver(queue=queue, max_running_jobs=self.max_running_jobs)

            # Saving state for plotting purposes
            # state_filename = "ExperimentStates/%s_%04i.pkl" % (self.experiment_name, loops)
            # saving_state(queue, state_filename)
            # saving_state(queue, "%s_state_nemo.pkl" % self.experiment_name)

            # if loops % 10 == 0:
            #     # checkpoint(queue, loops)
            #     checkpoint(queue)

            if wait:
                time.sleep(waiting_time)

        import pickle as pkl
        pkl.dump(whole_set.variable_set, open("%s_final_set.pkl" % self.experiment_name, "wb"))
